@@ -7,7 +7,7 @@ use std::{
 
 use atuin_common::utils;
 use clap::Subcommand;
-use eyre::Result;
+use eyre::{Context, Result};
 use runtime_format::{FormatKey, FormatKeyError, ParseSegment, ParsedFmt};
 
 use atuin_client::{
@@ -18,12 +18,12 @@ use atuin_client::{
 
 #[cfg(feature = "sync")]
 use atuin_client::sync;
-use log::debug;
+use log::{debug, warn};
+use time::{macros::format_description, OffsetDateTime};
 
-use super::search::format_duration;
 use super::search::format_duration_into;
 
-#[derive(Subcommand)]
+#[derive(Subcommand, Debug)]
 #[command(infer_subcommands = true)]
 pub enum Cmd {
     /// Begins a new command in the history
@@ -51,7 +51,18 @@ pub enum Cmd {
         #[arg(long)]
         cmd_only: bool,
 
-        /// Available variables: {command}, {directory}, {duration}, {user}, {host} and {time}.
+        /// Terminate the output with a null, for better multiline support
+        #[arg(long)]
+        print0: bool,
+
+        #[arg(long, short, default_value = "true")]
+        // accept no value
+        #[arg(num_args(0..=1), default_missing_value("true"))]
+        // accept a value
+        #[arg(action = clap::ArgAction::Set)]
+        reverse: bool,
+
+        /// Available variables: {command}, {directory}, {duration}, {user}, {host}, {exit} and {time}.
         /// Example: --format "{time} - [{duration}] - {directory}$\t{command}"
         #[arg(long, short)]
         format: Option<String>,
@@ -93,7 +104,13 @@ impl ListMode {
 }
 
 #[allow(clippy::cast_sign_loss)]
-pub fn print_list(h: &[History], list_mode: ListMode, format: Option<&str>) {
+pub fn print_list(
+    h: &[History],
+    list_mode: ListMode,
+    format: Option<&str>,
+    print0: bool,
+    reverse: bool,
+) {
     let w = std::io::stdout();
     let mut w = w.lock();
 
@@ -113,8 +130,22 @@ pub fn print_list(h: &[History], list_mode: ListMode, format: Option<&str>) {
         ListMode::CmdOnly => std::iter::once(ParseSegment::Key("command")).collect(),
     };
 
-    for h in h.iter().rev() {
-        match writeln!(w, "{}", parsed_fmt.with_args(&FmtHistory(h))) {
+    let iterator = if reverse {
+        Box::new(h.iter().rev()) as Box<dyn Iterator<Item = &History>>
+    } else {
+        Box::new(h.iter()) as Box<dyn Iterator<Item = &History>>
+    };
+
+    let entry_terminator = if print0 { "\0" } else { "\n" };
+    let flush_each_line = print0;
+
+    for h in iterator {
+        match write!(
+            w,
+            "{}{}",
+            parsed_fmt.with_args(&FmtHistory(h)),
+            entry_terminator
+        ) {
             Ok(()) => {}
             // ignore broken pipe (issue #626)
             Err(e) if e.kind() == io::ErrorKind::BrokenPipe => {
@@ -125,15 +156,28 @@ pub fn print_list(h: &[History], list_mode: ListMode, format: Option<&str>) {
                 std::process::exit(1);
             }
         }
+        if flush_each_line {
+            match w.flush() {
+                Ok(()) => {}
+                // ignore broken pipe (issue #626)
+                Err(e) if e.kind() == io::ErrorKind::BrokenPipe => {}
+                Err(err) => {
+                    eprintln!("ERROR: History output failed with the following error: {err}");
+                    std::process::exit(1);
+                }
+            }
+        }
     }
 
-    match w.flush() {
-        Ok(()) => {}
-        // ignore broken pipe (issue #626)
-        Err(e) if e.kind() == io::ErrorKind::BrokenPipe => {}
-        Err(err) => {
-            eprintln!("ERROR: History output failed with the following error: {err}");
-            std::process::exit(1);
+    if !flush_each_line {
+        match w.flush() {
+            Ok(()) => {}
+            // ignore broken pipe (issue #626)
+            Err(e) if e.kind() == io::ErrorKind::BrokenPipe => {}
+            Err(err) => {
+                eprintln!("ERROR: History output failed with the following error: {err}");
+                std::process::exit(1);
+            }
         }
     }
 }
@@ -141,6 +185,9 @@ pub fn print_list(h: &[History], list_mode: ListMode, format: Option<&str>) {
 /// type wrapper around `History` so we can implement traits
 struct FmtHistory<'a>(&'a History);
 
+static TIME_FMT: &[time::format_description::FormatItem<'static>] =
+    format_description!("[year]-[month]-[day] [hour repr:24]:[minute]:[second]");
+
 /// defines how to format the history
 impl FormatKey for FmtHistory<'_> {
     #[allow(clippy::cast_sign_loss)]
@@ -153,11 +200,17 @@ impl FormatKey for FmtHistory<'_> {
                 let dur = Duration::from_nanos(std::cmp::max(self.0.duration, 0) as u64);
                 format_duration_into(dur, f)?;
             }
-            "time" => self.0.timestamp.format("%Y-%m-%d %H:%M:%S").fmt(f)?,
+            "time" => {
+                self.0
+                    .timestamp
+                    .format(TIME_FMT)
+                    .map_err(|_| fmt::Error)?
+                    .fmt(f)?;
+            }
             "relativetime" => {
-                let since = chrono::Utc::now() - self.0.timestamp;
-                let time = format_duration(since.to_std().unwrap_or_default());
-                f.write_str(&time)?;
+                let since = OffsetDateTime::now_utc() - self.0.timestamp;
+                let d = Duration::try_from(since).unwrap_or_default();
+                format_duration_into(d, f)?;
             }
             "host" => f.write_str(
                 self.0
@@ -184,31 +237,29 @@ fn parse_fmt(format: &str) -> ParsedFmt {
 }
 
 impl Cmd {
+    #[allow(clippy::too_many_lines, clippy::cast_possible_truncation)]
     async fn handle_start(
-        db: &mut impl Database,
+        db: &impl Database,
         settings: &Settings,
         command: &[String],
     ) -> Result<()> {
         let command = command.join(" ");
 
-        if command.starts_with(' ') || settings.history_filter.is_match(&command) {
-            return Ok(());
-        }
-
         // It's better for atuin to silently fail here and attempt to
         // store whatever is ran, than to throw an error to the terminal
         let cwd = utils::get_current_dir();
-        if !cwd.is_empty() && settings.cwd_filter.is_match(&cwd) {
-            return Ok(());
-        }
 
         let h: History = History::capture()
-            .timestamp(chrono::Utc::now())
+            .timestamp(OffsetDateTime::now_utc())
             .command(command)
             .cwd(cwd)
             .build()
             .into();
 
+        if !h.should_save(settings) {
+            return Ok(());
+        }
+
         // print the ID
         // we use this as the key for calling end
         println!("{}", h.id);
@@ -217,7 +268,7 @@ impl Cmd {
     }
 
     async fn handle_end(
-        db: &mut impl Database,
+        db: &impl Database,
         settings: &Settings,
         id: &str,
         exit: i64,
@@ -226,7 +277,10 @@ impl Cmd {
             return Ok(());
         }
 
-        let mut h = db.load(id).await?;
+        let Some(mut h) = db.load(id).await? else {
+            warn!("history entry is missing");
+            return Ok(());
+        };
 
         if h.duration > 0 {
             debug!("cannot end history - already has duration");
@@ -236,7 +290,8 @@ impl Cmd {
         }
 
         h.exit = exit;
-        h.duration = chrono::Utc::now().timestamp_nanos() - h.timestamp.timestamp_nanos();
+        h.duration = i64::try_from((OffsetDateTime::now_utc() - h.timestamp).whole_nanoseconds())
+            .context("command took over 292 years")?;
 
         db.update(&h).await?;
 
@@ -255,14 +310,19 @@ impl Cmd {
         Ok(())
     }
 
+    #[allow(clippy::too_many_arguments)]
+    #[allow(clippy::fn_params_excessive_bools)]
     async fn handle_list(
-        db: &mut impl Database,
+        db: &impl Database,
         settings: &Settings,
         context: atuin_client::database::Context,
         session: bool,
         cwd: bool,
         mode: ListMode,
         format: Option<String>,
+        include_deleted: bool,
+        print0: bool,
+        reverse: bool,
     ) -> Result<()> {
         let session = if session {
             Some(env::var("ATUIN_SESSION")?)
@@ -276,7 +336,10 @@ impl Cmd {
         };
 
         let history = match (session, cwd) {
-            (None, None) => db.list(settings.filter_mode, &context, None, false).await?,
+            (None, None) => {
+                db.list(settings.filter_mode, &context, None, false, include_deleted)
+                    .await?
+            }
             (None, Some(cwd)) => {
                 let query = format!("select * from history where cwd = '{cwd}';");
                 db.query_history(&query).await?
@@ -293,12 +356,12 @@ impl Cmd {
             }
         };
 
-        print_list(&history, mode, format.as_deref());
+        print_list(&history, mode, format.as_deref(), print0, reverse);
 
         Ok(())
     }
 
-    pub async fn run(self, settings: &Settings, db: &mut impl Database) -> Result<()> {
+    pub async fn run(self, settings: &Settings, db: &impl Database) -> Result<()> {
         let context = current_context();
 
         match self {
@@ -309,10 +372,15 @@ impl Cmd {
                 cwd,
                 human,
                 cmd_only,
+                print0,
+                reverse,
                 format,
             } => {
                 let mode = ListMode::from_flags(human, cmd_only);
-                Self::handle_list(db, settings, context, session, cwd, mode, format).await
+                Self::handle_list(
+                    db, settings, context, session, cwd, mode, format, false, print0, reverse,
+                )
+                .await
             }
 
             Self::Last {
@@ -321,10 +389,13 @@ impl Cmd {
                 format,
             } => {
                 let last = db.last().await?;
+                let last = last.as_ref().map(std::slice::from_ref).unwrap_or_default();
                 print_list(
-                    &[last],
+                    last,
                     ListMode::from_flags(human, cmd_only),
                     format.as_deref(),
+                    false,
+                    true,
                 );
 
                 Ok(())
